/*
  Filename:	bdiff.c
  Author:	Ron Burkey
  Legal:	I, Ron Burkey, am placing this file in the public domain.
  Purpose:     	A program that simply compares two files byte by byte
  		and shows the differences.  It's intended to deal with the
		case where the files are the same size and only a few bytes
		such as a timestamp differ.
  Mod history:	2008-05-11 RSB	Created.
*/

#include <stdio.h>
#include <ctype.h>

int
main (int argc, char *argv[])
{
  FILE *fp1, *fp2;
  int i1, i2, Address, Last;
  char c1, c2;
  
  if (argc != 3)
    {
      printf ("Syntax:  bdiff file1 file2\n");
      return (1);
    }

  fp1 = fopen (argv[1], "r");
  if (fp1 == NULL)
    {
      printf ("File %s does not exist.\n", argv[1]);
      return (1);
    }
  fp2 = fopen (argv[2], "r");
  if (fp2 == NULL)
    {
      printf ("File %s does not exist.\n", argv[2]);
      return (1);
    }

  fseek (fp1, 0, SEEK_END);
  Last = ftell (fp1);
  fseek (fp2, 0, SEEK_END);
  if (Last != ftell (fp2))
    {
      printf ("Files are not the same size.\n", argv[1]);
      return (1);
    }

  fseek (fp1, 0, SEEK_SET);
  fseek (fp2, 0, SEEK_SET);
  for (Address = 0; Address < Last; Address++)
    {
      i1 = fgetc (fp1);
      i2 = fgetc (fp2);
      if (i1 != i2)
        {
	  c1 = i1;
	  if (!isprint (c1))
	    c1 = '.';
	  c2 = i2;
	  if (!isprint (c2))
	    c2 = '.';
	  printf ("0x%08X: 0x%02X \'%c\' != 0x%02X \'%c\'\n", Address, i1, c1, i2, c2);
	}
    }

  return (0);
}

